(*
   This file is an example for the ParsingToolkit project -
   it implements a parser for a small subset of RenderMan RIB files.

   Copyright (C) 1999 ANOQ of the Sun (alias Johnny Andersen)

   E-mail: anoq@HardcoreProcessing.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   If you are interested in a warranty or commercial support for this
   software, contact Hardcore Processing <sales@HardcoreProcessing.com>
   for more information.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

functor FRenderManRIBParse() =
  struct
    open TextIOParserCombinators
    open TextIOParserCombExtra

    val getP = ($$ "\"P\"") --$ whitespace

    val getLeftSqBracketWS = ($$ "[") --$ whitespace
    val getRightSqBracketWS = ($$ "]") --$ whitespace

    val getReals = repeat getRealWS
    val getRealList = getLeftSqBracketWS $-- getReals --$ getRightSqBracketWS

    fun getPoint charStream =
          let
            val (((x, y), z), charStream1) =
                   ((getRealWS -- getRealWS) -- getRealWS) charStream
          in
            ({x = x, y = y, z = z}, charStream1)
          end

    val getPointWS = getPoint --$ whitespace

    val getPoints = repeat getPointWS
    val getPointList = getLeftSqBracketWS $--
                       getPoints
                       --$ getRightSqBracketWS

    fun parsePointsPolygons charStream =
          let
            (* Notice: I know that nverts and vertids should really be
                       integers, but to keep the example small... *)
            val (((nverts, vertids), vertices), charStream1) =
                    (((whitespaceForce $-- getRealList) -- (getRealList --$ getP)) -- getPointList) charStream
          in
            ((nverts, vertids, vertices), charStream1)
          end

    fun startImport charStream =
          let
            val (_, charStream1) =
                   $$ "PointsPolygons" charStream
          in
            parsePointsPolygons charStream1
          end
            handle SyntaxError _ =>
              (case FuncTextIO.input1 charStream of
                 NONE =>
                   raise SyntaxError ("Could not find PointsPolygons command", charStream)
               | SOME(_, charStream2) =>
                   startImport charStream2)

    exception ImportError

    fun import fileName =
          let
            val inStream = FuncTextIO.openIn fileName

            val (v, _) = (startImport inStream)
                            handle SyntaxError (str, _) =>
                              (print str; raise ImportError)

            val _ = FuncTextIO.closeIn inStream
          in
            v
          end
  end

structure RenderManRIBParse = FRenderManRIBParse()