(*
    This file is part of the ParsingToolkit project -
    which provides combinator parsers for functional input streams.
    
    Copyright (C) 2000 ANOQ of the Sun (alias Johnny Andersen).
    
    Authors: Fritz Henglein <henglein@it.edu>
             ANOQ of the Sun (alias Johnny Andersen)
               <anoq@HardcoreProcessing.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    As a special exception, if you do not do anything which is not in
    the spirit of the GNU Library General Public License, you are not
    required to physically compile this software into a separate library,
    since this is generally not possible with current Stanard ML compilers.
    However if you do something which is not in the spirit of the
    GNU Library General Public License you will have to follow the
    licence perpetually - thus disallowing you to use it for any
    commercial purposes at all.

    If you are interested in a warranty or commercial support for this
    software, contact Hardcore Processing <sales@HardcoreProcessing.com>
    for more information.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

(* Built upon Fritz Henglein's implementation of
   parser combinators as found in Larry Paulson's
   "ML for the Working Programmer"  *)

functor FTextIOParserCombExtra() :> TEXT_IO_PARSER_COMB_EXTRA =
  struct
    open TextIOParserCombinators
    
    (* Character predicates *)

    fun isWhitespaceChar #" " = true
      | isWhitespaceChar #"\009" = true
      | isWhitespaceChar #"\r" = true
      | isWhitespaceChar #"\n" = true
      | isWhitespaceChar _ = false

    val isLetterChar = Char.isAlpha (* must be generalized! *)
    val isDigitChar = Char.isDigit

    (* Lexical scanner functions *)
    val whitespaceForce =
          repeatOneIf isWhitespaceChar

    val whitespace =
          repeatIf isWhitespaceChar

    fun getReal stream =
          let
            val (digits, stream1) =
                   repeatOneIf (fn c => isDigitChar c orelse
                                        c = #"." orelse
                                        c = #"-")
                               stream
            val realChars = map (fn c => if c = #"-" then #"~" else c) digits
            val realStr = implode realChars
          in
            case Real.fromString realStr of
              NONE =>
                raise SyntaxError ("The string " ^ realStr ^ " is not a real number", stream1)
            | SOME r =>
                (r, stream1)
          end

    val getRealWS = getReal --$ whitespace
  end

structure TextIOParserCombExtra = FTextIOParserCombExtra()